$	if f$type(omi$_debug) .eqs. "" then $ omi$_debug = 0
$	if .not. omi$_debug then $ omi$_verify = f$verify(0)
$	on control_y then $ goto main$_interrupt
$	on error then $ goto main$_fatal
$	on severe_error then $ goto main$_fatal
$	omi$_control = f$environment("control")
$	omi$_message = f$environment("message")
$	goto main$_start
$!
$!******************************************************************************
$!*                                                                            *
$!*  FILENAME:                                                                 *
$!*  =========                                                                 *
$!*     Omi$Menu.Com       Oscar's Menu Interface                              *
$!*                                                                            *
$!* ************************************************************************** *
$!* *                                                                        * *
$!* * 1997, Oscar van Eijk - OVEAS                                          * *
$!* *   This tool is delivered as is, and has no warranties whatsoever.      * *
$!* *   It may be freely distributed as long as the distribution set is      * *
$!* *   complete. It is not allowed to change any of the files, without      * *
$!* *   permission of the author.                                            * *
$!* *                                                                        * *
$!* ************************************************************************** *
$!*                                                                            *
$!*  DESCRIPTION:                                                              *
$!*  ============                                                              *
$!*                                                                            *
$!*      Oscar's Menu Interface is a DCL procedure that reads menu files and   *
$!*      represents the menu structure on a ChUI based window.                 *
$!*                                                                            *
$!*      OMI does not perform any actions on its own. Additional procedures    *
$!*      are required to handle upon the users input. OMI is strictly created  *
$!*      to create a standard user interface for all kinds of actions, with a  *
$!*      clear structure, support for different security levels and password   *
$!*      protection.                                                           *
$!*                                                                            *
$!*      The additional procedures that are required for any actions are       *
$!*      referred to 'callable jobs'. A section in the help file describes     *
$!*      how several commands, defined in OMI, can be used to create such      *
$!*      procedures.                                                           *
$!*                                                                            *
$!*  FILES NEEDED:                                                             *
$!*  =============                                                             *
$!*     Omi$Config.Com        The procedure that sets up the configuration     *
$!*                           and the menu environments by reading the files   *
$!*                           and defining global symbols. On exit, this       *
$!*                           procedure also cleans up all symbols.            *
$!*     Omi$Screen.Com        All screen related material is handled by this   *
$!*                           procedure. The window and layout that's defined, *
$!*                           is based upon the settings by Omi$Config.Com.    *
$!*     Omi$ToolBox.Com       A set of routines that will are called by OMI,   *
$!*                           but which are also available for OMI modules.    *
$!*                           Most OMI commands call routines from this        *
$!*                           procedure.                                       *
$!*     Omi$ToolBox.Ini       Initialisation file for the ToolBox. All         *
$!*                           that are available via the ToolBox should be     *
$!*                           added here. This file can also define additional *
$!*                           ToolBox files.                                   *
$!*     Omi$Menu.Cfg          This is the configuration file in which  the     *
$!*                           layout can be defined, and many option of the    *
$!*                           behaviour can be changed. It should be located in*
$!*                           SYS$LOGIN of the current user. If not there,     *
$!*                           the procedure looks for the  file in OMI$.       *
$!*                           If the logical OMI$CONFIG is defined, this over- *
$!*                           writes all other files.                          *
$!*     <file>.MNU            One or more MNU files can be created, containing *
$!*                           the menus. The procedure looks for the MNU files *
$!*                           in OMI$MENU_DIRECTORY, if set, and in OMI$       *
$!*                                                                            *
$!*  LOGICALS NEEDED:                                                          *
$!*  ================                                                          *
$!*     OMI$                  This logical points to the directory where this  *
$!*                           procedure, and its subprocedures, is located.    *
$!*                           It's also part of the search-path for the CFG    *
$!*                           file, and the MNU files. When not set, it's done *
$!*                           dynamically.                                     *
$!*     OMI$MENU_DIRECTORY    If set, this points to the default directory     *
$!*                           where all menu files are stored.                 *
$!*     OMI$CONFIG            This logical can point to the configuration file.*
$!*                           When it's not set, OMI looks for the file        *
$!*                           Omi$Menu.Cfg in SYS$LOGIN first, then in OMI$    *
$!*     OMI$STARTMENU         If defined, this is the menu file with which the *
$!*                           procedure starts.                                *
$!*                                                                            *
$!*  PARAMETERS NEEDED:                                                        *
$!*  ==================                                                        *
$!*     P1                    If this parameter is specified, it should point  *
$!*                           to the menu file to start with.                  *
$!*     P2 - P3               If specified, P2 contains the name of the menu   *
$!*                           which will be started first, and P3 selects an   *
$!*                           option in the specified menu.                    *
$!*                           If the parameters are used, P1 is required.      *
$!*                                                                            *
$!*  HISTORY:                                                                  *
$!*  ========                                                                  *
$!*     Version:  Date:       Author:                 Changes:                 *
$!*     --------  -----       -------                 --------                 *
$!*     0.0       11-03-1997  Oscar van Eijk, OVEAS   Initial version,         *
$!*                    containing submenus, commands with input option,        *
$!*                    free inputs and selection lists.                        *
$!*     1.0b1     14-03-1997  Oscar van Eijk, OVEAS   Additions:               *
$!*                    dynamic menu's, counters, OMI commands SET, RESET and   *
$!*                    SHOW, implemented the version number, the comment       *
$!*                    field and the password option.                          *
$!*     1.0b2     15-04-1997  Oscar van Eijk, OVEAS   Additions:               *
$!*                    Type defs. for the input items, tag lists, the CALL     *
$!*                    item- type and several commands that can be used by     *
$!*                    called procedures.                                      *
$!*                    Added the free option in select lists.                  *
$!*     1.0b3     09-05-1997  Oscar van Eijk, OVEAS   Additions:               *
$!*                    Fixed a security bug; a security block in the main      *
$!*                    menu wasn't properly handled.                           *
$!*                    Added the QUIT command as a synonym for EXIT.           *
$!*                    Added extra parameters to select a submenu by name and  *
$!*                    the options immediatly from the DCL command line when   *
$!*                    starting OMI, and the JUMP command.                     *
$!*                    Moved most extra options (like the encryption module    *
$!*                    and the OMI$CHECK command) to the newly created         *
$!*                    OMI$TOOLBOX.COM.                                        *
$!*                    Added the commands OMI$CREATE_MBX, OMI$ENCRYPT,         *
$!*                    OMI$DECRYPT and OMI$REVERSE.                            *
$!*                    Added the FILESPEC format type.                         *
$!*                    Enhanced the encryption method, which is incompatible   *
$!*                    with the encryption of v1.0b2.                          *
$!*                     *NOTE* Due to this, password checks will now fail if   *
$!*                            the password was set with an older release.     *
$!*                            The password value needs to be removed manually *
$!*                            from the MNU file, after which it will default  *
$!*                            to 'Omi$System'. It can then be set again with  *
$!*                            the SET PASSWORD command.                       *
$!*                    Added the '<EOF>' identifier for menu and config files. *
$!*                    Added the ENCRYPT command.                              *
$!*     1.0b4     13-05-1997  Oscar van Eijk, OVEAS   Additions:               *
$!*                    Bugfix for the DCL-W-UNDSYM error on password protected *
$!*                    menus.                                                  *
$!*                    Implemented the ASK command.                            *
$!*                    Implemented the OMI$INPUT_VALIDATE command, that fixes  *
$!*                    a possible bug when a user enteres ''F$PID(GOTO)' as    *
$!*                    input.                                                  *
$!*                    Thanks to Henry Juengst (juengst@saph1.iskp.uni-bonn.de)*
$!*                    for making me aware of this.                            *
$!*     1.0       30-05-1997  Oscar van Eijk, OVEAS   Additions:               *
$!*                    Added the option to specify a parameter for             *
$!*                    OMI$DATE_INFO, which enables date information for other *
$!*                    dates. Default is 'today'.                              *
$!*                    Fixed a bug in OMI$DATE_INFO, which miscalculates       *
$!*                    after september.                                        *
$!*                    Some minor documentation bugs in the HELP file fixed.   *
$!*                                                                            *
$!******************************************************************************

$!******************************************************************************
$!
$!==>	These initials settings setup the menu environment by calling all
$!	proper subroutines and -procedures.
$!	The first menu will allways be called 'menu'.
$!
$ main$_start:
$!
$	ws := "write sys$output"
$	cls := "type/page nla0:"
$	omi$version = "1.0, Released May 30, '97"
$	omi$version_id = "1"   ! Used to check the config- file
$	gosub main$_initialize
$	gosub main$_getstart
$	omi$menu_file = "''omi$startmenu'"
$!
$ main$_startmenu:
$!
$	omi$display_message "%OMI-I-INIT, initializing..."
$	omi$config 'omi$menu_file
$	omi$status = $status
$	omi$cmdline_clear
$	if omi$status .eq. omi$_warning
$	   then
$!
$	 main$_askfor_start:
$!
$		read /end_of_file=main$_exit sys$command omi$startmenu -
		   /prompt="[''screen$line_command';''screen$default_position'HMenu file: " 
$		omi$cmdline_clear
$		omi$msgline_clear
$		omi$variable = "omi$startmenu"
$		omi$input_validate
$		if $status .ge. omi$_warning
$		   then
$			omi$display_message "%OMI-E-TRANERR, cannot translate input - invalid variable"
$			goto main$_askfor_start
$		endif
$		omi$menu_file = "''omi$startmenu'"
$		goto main$_startmenu
$	endif
$	if omi$status .eq. omi$_error then $ goto main$_fatal
$	omi$current_menu = "menu"
$	'omi$current_menu'$previous = ""
$	gosub main$default_values
$	gosub main$check_security
$	if 'omi$current_menu'$security_level .eq. -1
$	   then
$		omi$display_message "%OMI-W-IVPWD, invalid password - access denied"
$		goto main$_interrupt
$	endif
$	if 'omi$current_menu'$security_level .eq. 0
$	   then
$		omi$display_message "%OMI-W-NOTAUTH, you are not authorized for this menu - exiting"
$		goto main$_interrupt
$	endif
$!
$	omi$msgline_clear
$!
$	omi$display_message "%OMI-I-INFO, Oscar's Menu Interpreter, 1997, O. van Eijk - OVEAS"
$	if f$type(menu$on_init) .nes. ""
$	   then
$		omi$init_job = menu$on_init
$		if f$locate(".",omi$init_job) .eq. f$length(omi$init_job) -
		   then $ omi$init_job = omi$init_job + ".OMI"
$		@'omi$init_job
$	endif
$	if p2 .nes. ""
$	   then
$		omi$_jumping = 1
$		p3$_counter = 0
$		omi$_p1 = p2
$		jump$_norefresh = 1
$		gosub main$execcmd_jump
$	   else $ omi$_jumping = 0
$	endif
$	goto main$do_menu
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	We've found and initialized a menu. Display it, and start prompting
$!	the user to navigate through the options.
$!
$ main$do_menu:
$!
$	omi$screen menu
$!
$ main$get_option:
$!
$	if omi$_jumping
$	   then
$		_option = f$element(p3$_counter,",",p3)
$		if _option .eqs. "" .or. _option .eqs. ","
$		   then
$			read /end_of_file=option$cancel_input /prompt="[''screen$line_command';''screen$default_position'H''questions$option': " sys$command _option
$			if f$type(jump$_norefresh) .nes. "" then -
			   $ deletee /symbol /local jump$_norefresh
$			omi$_jumping = 0
$		   else $ p3$_counter = p3$_counter + 1
$		endif
$	   else $ read /end_of_file=option$cancel_input /prompt="[''screen$line_command';''screen$default_position'H''questions$option': " sys$command _option
$	endif
$	omi$variable = "_option"
$	omi$input_validate
$	if $status .ge. omi$_warning
$	   then
$		omi$display_message "%OMI-E-TRANERR, cannot translate input - invalid variable"
$		omi$cmdline_clear
$		goto main$get_option
$	endif
$	omi$cmdline_clear
$	omi$msgline_clear
$	if _option .eqs. "" then $ goto main$get_option
$!
$	if 'omi$current_menu'$security_level .eq. 1
$	   then
$		omi$display_message "%OMI-W-NOTAUTH, not authorized - use <Ctrl/Z> to go back"
$		goto main$get_option
$	endif
$!
$	if f$type(_option) .eqs. "INTEGER"
$	   then
$		if _option .eq. 0 then $ goto option$cancel_input
$		if f$type('omi$current_menu'$item'_option') .eqs. ""
$		   then
$			_input = _option - inputs$highest_item
$			if f$type('omi$current_menu'$input'_input') .eqs. ""
$			   then
$				omi$display_message "%OMI-W-IVOPT, option is out of range"
$				goto main$get_option
$			   else
$				gosub main$askfor_input
$				goto main$get_option
$			endif
$		endif
$		goto main$option_eval
$	endif
$!
$	omi$_command = f$edit(_option, "upcase")
$	gosub main$omi_command
$	if f$type(omi$previous_menu_file) .nes. ""
$	   then
$		deletee/symbol/local omi$previous_menu_file
$		goto main$_startmenu
$	endif
$	goto main$get_option
$!
$ option$cancel_input:
$!
$	if 'omi$current_menu'$previous .eqs. ""
$	   then
$		gosub main$_exit
$		goto main$do_menu
$	endif
$	omi$current_menu = 'omi$current_menu'$previous
$	omi$cmdline_clear
$	omi$msgline_clear
$	goto main$do_menu
$! 
$!******************************************************************************

$!******************************************************************************
$!
$!==>	If the current menu contains input options, this routine asks the
$!	user for input.
$!
$ main$askfor_input:
$!
$	_all_inputs = 0
$	_variable = f$element(1,"#",'omi$current_menu'$input'_input')
$!
$	if f$extract(0,5,f$edit(_variable,"upcase")) .eqs. "{TAG|"
$	   then
$		gosub main$_taglist
$		return
$	endif
$!
$	_sel_list = (f$extract(0,1,_variable) .eqs. "{")
$!
$	if _sel_list
$	   then
$		_select_list = f$extract(0, f$locate("}", _variable) + 1, _variable)
$		_variable = _variable - _select_list
$		_select_list = f$edit(_select_list,"upcase") - "{SEL|" - "}"
$		omi$screen select_list
$	endif
$!
$ input$_prompt:
$!
$	if omi$_jumping
$	   then
$		'_variable' = f$element(p3$_counter,",",p3)
$		if '_variable' .eqs. "" .or. '_variable' .eqs. ","
$		   then
$			read /end_of_file=input$cancel_input /prompt="[''screen$line_command';''screen$default_position'H''questions$input': " sys$command '_variable'
$			if f$type(jump$_norefresh) .nes. "" then -
			   $ deletee /symbol /local jump$_norefresh
$			omi$_jumping = 0
$			omi$msgline_clear
$		   else $ p3$_counter = p3$_counter + 1
$		endif
$	   else
$		read /end_of_file=input$cancel_input /prompt="[''screen$line_command';''screen$default_position'H''questions$input': " sys$command '_variable'
$		omi$msgline_clear
$	endif
$!
$	omi$variable = "''_variable'"
$	omi$input_validate
$	if $status .ge. omi$_warning
$	   then
$		omi$display_message "%OMI-E-TRANERR, cannot translate input - invalid variable"
$		omi$cmdline_clear
$		goto input$_prompt
$	endif
$	_format = f$element(3, "#",'omi$current_menu'$input'_input') 
$	if _format .nes. "" .and. _format .nes. "#"
$	   then
$		gosub input$_format
$		omi$cmdline_clear
$		if .not. _frm_okay
$		   then
$			if f$type(jump$_norefresh) .nes. "" then -
			   $ deletee /symbol /local jump$_norefresh
$			omi$_jumping = 0
$			goto input$_prompt
$		endif
$	endif
$	if _sel_list
$	   then
$		_selected = '_variable'
$		if f$type(_selected) .nes. "INTEGER" .or. -
		   f$type('_select_list'$value'_selected') .eqs. ""
$		   then
$			omi$display_message "%OMI-W-IVCHOICE, please specify a valid choice"
$			omi$cmdline_clear
$			if f$type(jump$_norefresh) .nes. "" then -
			   $ deletee /symbol /local jump$_norefresh
$			omi$_jumping = 0
$			goto input$_prompt
$		endif
$		'_variable' = '_select_list'$value'_selected'
$		if f$extract(0, 1, '_variable') .eqs. "{"
$		   then
$			_prompt = '_variable' - "{" - "}"
$!
$		 sellist$get_free_input:
$!
$			omi$cmdline_clear
$			read /end_of_file=input$cancel_input /prompt="[''screen$line_command';''screen$default_position'H''_prompt': " sys$command '_variable'
$			omi$variable = "''_variable'"
$			omi$input_validate
$			if $status .ge. omi$_warning
$			   then
$				omi$display_message "%OMI-E-TRANERR, cannot translate input - invalid variable"
$				goto sellist$get_free_input
$			endif
$		endif
$		omi$refresh
$	endif
$	_line = inputs$first_line - 1 + '_input'
$	_value = '_variable'
$	_blanks = inputs$max_size - f$length(_value) + 1
$	if f$length(_value) .le. inputs$max_size
$	   then $ ws f$fao("[''_line';''inputs$value_location'H''_value'!''_blanks'* ")
$	   else $ ws "[''_line';''inputs$value_location'H''f$extract(0,inputs$max_size,_value)'(0`(B"
$	endif
$	omi$cmdline_clear
$	omi$msgline_clear
$	return
$!
$ input$cancel_input:
$!
$	if _sel_list
$	   then $ omi$refresh
$	   else
$		omi$cmdline_clear
$		omi$msgline_clear
$	endif
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	The 4th argument of the input item in the menu file can specify a block
$!	that will be called here for string formatting.
$!	The string
$!
$ input$_format:
$!
$	on warning then $ goto input$invalid_format
$	if .not. omi$_debug then -
	   $ set message /nofacility /noseverity /noidentification /notext
$	_frm_okay = 0
$	_req_format = f$edit('_format'$type,"collapse,upcase")
$	goto input$'_req_format'_format
$!
$!******************************************************************************
$!
$!==>	Validate the STRING type
$!
$ input$string_format:
$!
$	if f$type('_variable') .nes. "STRING" then $ goto input$invalid_format
$!
$	if f$type('_format'$upcase) .eqs. ""
$	   then $ _upcase = "false"
$	   else
$		if '_format'$upcase
$		   then $ _upcase = "true"
$		   else $ _upcase = "false"
$		endif
$	endif
$!
$	if f$type('_format'$lowercase) .eqs. ""
$	   then $ _lowercase = "false"
$	   else
$		if '_format'$lowercase
$		   then $ _lowercase = "true"
$		   else $ _lowercase = "false"
$		endif
$	endif
$!
$	if f$type('_format'$blanks) .eqs. ""
$	   then $ _collapse = "false"
$	   else
$		if '_format'$blanks
$		   then $ _collapse = "false"
$		   else $ _collapse = "true"
$		endif
$	endif
$!
$	if _upcase    then $ '_variable' = f$edit('_variable',"upcase")
$	if _lowercase then $ '_variable' = f$edit('_variable',"lowercase")
$	if _collapse  then $ '_variable' = f$edit('_variable',"collapse")
$!
$	if f$type('_format'$minlength) .nes. ""
$	   then
$		if f$length('_variable') .lt. f$integer('_format'$minlength)
$		   then
$			on warning then $ continue
$			set message 'omi$_message'
$			omi$display_message "%OMI-W-IVLEN, string length is not allowed to be less than ",'_format'$minlength
$			return
$		endif
$	endif
$!
$	if f$type('_format'$maxlength) .nes. ""
$	   then
$		if f$length('_variable') .gt. f$integer('_format'$maxlength)
$		   then
$			on warning then $ continue
$			set message 'omi$_message'
$			omi$display_message "%OMI-W-IVLEN, string length is not allowed to be more than ",'_format'$maxlength
$			return
$		endif
$	endif
$!
$	if f$type('_format'$ivchars) .eqs. "" then $ goto input$end_format
$!
$	_char_c = 0
$!
$ format$stringchars:
$!
$	_char = f$extract(_char_c, 1, '_format'$ivchars)
$	if f$locate(_char, '_variable') .lt. f$length('_variable')
$	   then
$		on warning then $ continue
$		set message 'omi$_message'
$		omi$display_message "%OMI-W-IVCHAR, invalid character encoutered - \''_char'\ not allowed"
$		return
$	endif
$!
$	_char_c = _char_c + 1
$	if _char_c .lt. f$length('_format'$ivchars) then $ goto format$stringchars
$	goto input$end_format
$!
$!******************************************************************************
$!
$!==>	Validate the FILESPEC type
$!
$ input$filespec_format:
$!
$	if f$type('_variable') .nes. "STRING" then $ goto input$invalid_format
$!
$	if f$type('_format'$wildcards) .eqs. ""
$	   then $ _allow_wc = "false"
$	   else
$		if '_format'$wildcards
$		   then $ _allow_wc = "true"
$		   else $ _allow_wc = "false"
$		endif
$	endif
$!
$	if .not. _allow_wc .and. (f$locate("%", '_variable') .lt. -
	   f$length('_variable') .or. f$locate("*", '_variable') .lt. -
	   f$length('_variable'))
$	   then
$		on warning then $ continue
$		set message 'omi$_message'
$		omi$display_message "%OMI-W-NOWILDCARD, wildcards are not allowed in this option"
$		return
$	endif
$!
$	'_variable' = f$edit('_variable',"upcase,collapse")
$	if f$parse('_variable',,,,"syntax_only") .eqs. ""
$	   then
$		on warning then $ continue
$		set message 'omi$_message'
$		omi$display_message "%OMI-W-IVFNAM, invalid file or device name \",'_variable',"\"
$		return
$	endif
$!
$	if f$type('_format'$required) .eqs. ""
$	   then $ _existreq = "false"
$	   else
$		if '_format'$required
$		   then $ _existreq = "true"
$		   else $ _existreq = "false"
$		endif
$	endif
$!
$	if _existreq
$	   then
$		_excheck = '_variable'
$		if f$extract(f$length(_excheck) - 1, 1, _excheck) .eqs. "]" -
		   then $ _excheck = _excheck + "*.*"
$		if f$search(_excheck) .eqs. ""
$		   then
$			on warning then $ continue
$			set message 'omi$_message'
$			omi$display_message "%OMI-W-FNF, file \",'_variable',"\ does not exist"
$			return
$		endif
$	endif
$	goto input$end_format
$!
$!******************************************************************************
$!
$!==>	Validate the INTEGER type
$!
$ input$integer_format:
$!
$	if f$type('_variable') .nes. "INTEGER" then $ goto input$invalid_format
$!
$	if f$type('_format'$min) .nes. ""
$	   then
$		if '_variable' .lt. '_format'$min
$		   then
$			on warning then $ continue
$			set message 'omi$_message'
$			omi$display_message "%OMI-W-LOWVAL, value can't be less than ",'_format'$min
$			return
$		endif
$	endif
$!
$	if f$type('_format'$max) .nes. ""
$	   then
$		if '_variable' .gt. '_format'$max
$		   then
$			on warning then $ continue
$			set message 'omi$_message'
$			omi$display_message "%OMI-W-HIVAL, value can't be more than ",'_format'$max
$			return
$		endif
$	endif
$!
$	goto input$end_format
$!
$!******************************************************************************
$!
$!==>	Validate the DATETIME type
$!
$ input$datetime_format:
$!
$	omi$display_message "%OMI-I-NOTYET, format ''_req_format' not yet implemented"
$	goto input$end_format
$!
$!******************************************************************************
$!
$!==>	Validate the DATE type
$!
$ input$date_format:
$!
$	if f$type('_format'$format) .eqs. ""
$	   then
$		_dformat = "absolute"
$		goto format$frm_date
$	endif
$!
$	if f$edit(f$extract(0, 1, '_format'$format), "upcase") .eqs. "A" then -
	   $ _dformat = "absolute"
$	if f$edit(f$extract(0, 1, '_format'$format), "upcase") .eqs. "C" then -
	   $ _dformat = "comparison"
$	if f$edit(f$extract(0, 1, '_format'$format), "upcase") .eqs. "D" then -
	   $ _dformat = "delta"
$!
$ format$frm_date:
$!
$	_d_in = '_variable'
$	if f$locate("-",_d_in) .eq. f$length(_d_in) then -
	   $ _d_in = _d_in + "-" + f$cvtime("today","absolute","month")
$	'_variable' = f$cvtime("''_d_in'","''_dformat'","date")
$	goto input$end_format
$!
$!******************************************************************************
$!
$!==>	Validate the TIME type
$!
$ input$time_format:
$!
$	omi$display_message "%OMI-I-NOTYET, format ''_req_format' not yet implemented"
$	goto input$end_format
$!
$ input$end_format:
$!
$	_frm_okay = 1
$	set message 'omi$_message'
$	on warning then $ continue
$	return
$!
$ input$invalid_format:
$!
$	on warning then $ continue
$	set message 'omi$_message'
$	omi$display_message "%OMI-W-IVFRM, invalid format, specify ''_req_format' type"
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	An option has been entered and validated. This routine translates
$!	the option to the menu definition and acts upon that.
$!
$ main$option_eval:
$!
$	_selected_item = 'omi$current_menu'$item'_option'
$!
$ eval$lookfor_substitutions:
$!
$	if f$locate("~?",_selected_item) .lt. f$length(_selected_item)
$	   then
$		gosub main$input_substitute
$		goto eval$lookfor_substitutions
$	endif
$!
$	if f$edit(f$element(1,"#",_selected_item),"upcase") .eqs. "SUBMENU"
$	   then
$		if f$extract(0, 1, f$element(2,"#",_selected_item)) .eqs. "{"
$		   then $ gosub main$_dynamic_menu
$		   else
$			'f$element(2,"#",_selected_item)'$previous = omi$current_menu
$			omi$current_menu = "''f$element(2,"#",_selected_item)'"
$		endif
$		gosub main$check_security
$		if 'omi$current_menu'$security_level .eq. -1
$		   then
$			omi$display_message "%OMI-W-IVPWD, invalid password - access denied"
$			omi$current_menu = 'omi$current_menu'$previous
$		endif
$		if 'omi$current_menu'$security_level .eq. 0
$		   then
$			omi$display_message "%OMI-W-NOPRIV, you are not authorized for this menu"
$			omi$current_menu = 'omi$current_menu'$previous
$		endif
$		if f$type('omi$current_menu'$counter) .nes. ""
$		   then
$			_menu_counter = 'omi$current_menu'$counter
$			counter$'_menu_counter' == counter$'_menu_counter' + 1
$		endif
$		goto main$do_menu
$	endif
$!
$	_selected_item = f$edit(_selected_item,"upcase")
$!
$	if f$element(1,"#",_selected_item) .eqs. "COMMAND"
$	   then
$		omi$_command = f$element(2,"#",_selected_item)
$		gosub main$omi_command
$		if f$type(omi$previous_menu_file) .nes. ""
$		   then
$			deletee/symbol/local omi$previous_menu_file
$			goto main$_startmenu
$		endif
$		goto main$get_option
$	endif
$!
$	if f$element(1,"#",_selected_item) .eqs. "CALL"
$	   then
$		omi$call_info = f$edit(f$element(2,"#",_selected_item), -
		   "trim,compress")
$		if f$extract(0,1,omi$call_info) .eqs. "@" then -
		   $ omi$call_info = omi$call_info - "@"
$		omi$job_call  = f$element(0," ",omi$call_info)
$		omi$call_parm = f$edit((omi$call_info - omi$job_call),"trim")
$!		omi$job_call  = f$parse(omi$job_call,".OMI",,,"syntax_only")
$! The above line is outcommented; it doesn't work when there's a variable
$! in the filename (eg. 'DIRECTORIES$OMI_JOBS'MY_MODULE)
$! Below is the wordaround.
$		if f$locate(".",omi$job_call) .eq. f$length(omi$job_call) -
		   then $ omi$job_call = omi$job_call + ".OMI"
$		if .not. omi$_debug then -
		   $ set message /nofacility /noseverity /noidentification /notext
$		@'omi$job_call' 'omi$call_parm'
$		_status = $status
$		set message 'omi$_message
$		goto main$get_option
$	endif
$!
$	omi$display_message "%OMI-E-BADOPT, bad option - check the menu file"
$	goto main$get_option
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	Handle the Dynamic Menu items
$!
$ main$_dynamic_menu:
$!
$	_menu_list = f$element(2,"#",_selected_item)
$	_dynmenu_count = 0
$!
$ dynmenu$_get_options:
$!
$	_dynmenu_count = _dynmenu_count + 1
$	_dynmenu'_dynmenu_count' = f$extract(0, f$locate("}", _menu_list)+1, _menu_list)
$	_menu_list = _menu_list - _dynmenu'_dynmenu_count'
$	_dynmenu'_dynmenu_count' = _dynmenu'_dynmenu_count' - "{" - "}"
$	if f$extract(0, 1, _menu_list) .eqs. "{" then $ goto dynmenu$_get_options
$	omi$screen dynamic_menu
$!
$ dynmenu$_prompt:
$!
$	read /end_of_file=dynmnu$cancel_input /prompt="[''screen$line_command';''screen$default_position'H''_menu_list' " sys$command _selected_menu
$	omi$variable = "_selected_menu"
$	omi$input_validate
$	if $status .ge. omi$_warning
$	   then
$		omi$display_message "%OMI-E-TRANERR, cannot translate input - invalid variable"
$		omi$cmdline_clear
$		goto dynmenu$_prompt
$	endif
$	if f$type(_selected_menu) .nes. "INTEGER" .or. -
 	   f$type(_dynmenu'_selected_menu') .eqs. ""
$	   then
$		omi$display_message "%OMI-W-IVCHOICE, please specify a valId choice"
$		omi$cmdline_clear
$		goto dynmenu$_prompt
$	endif
$	_selected_menu = f$element(1, "|", _dynmenu'_selected_menu')
$	'_selected_menu'$previous = omi$current_menu
$	omi$current_menu = "''_selected_menu'"
$!
$ dynmnu$cancel_input:
$!
$	omi$cmdline_clear
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	An OMI command was found in the menu file, which requires extra
$!	input. This was recorgnized by the string '~?' If this was immediatly
$!	followed by '{any text}', 'any text' will be used to prompt the user
$!	for the required input. '~?' will be substituted by the user input in
$!	the command.
$!
$ main$input_substitute:
$!
$	_string1 = f$extract(0, f$locate("~?",_selected_item), _selected_item)
$	_selected_item = _selected_item - _string1
$!
$	if f$locate("~?{",_selected_item) .lt. f$length(_selected_item)
$	   then
$		_prompt = f$extract(3, f$locate("}",_selected_item)-3, _selected_item)
$		_selected_item = _selected_item - "{''_prompt'}"
$	   else $ _prompt = "''questions$default_input':"
$	endif
$!
$ main$input_required_bug:
$!
$	read /end_of_file=main$input_bug sys$command _value -
 	   /prompt="[''screen$line_command';''screen$default_position'H''_prompt' " 
$	omi$variable = "_value"
$	omi$input_validate
$	if $status .ge. omi$_warning
$	   then
$		omi$display_message "%OMI-E-TRANERR, cannot translate input - invalid variable"
$		omi$cmdline_clear
$		goto main$input_required_bug
$	endif
$	omi$cmdline_clear
$	omi$msgline_clear
$!
$	_selected_item = _string1 + _value + f$extract(2, -
 	   f$length(_selected_item)-2, _selected_item)
$!
$	return
$!
$ main$input_bug:	! Don't know how to handle <Cntrl/Z> here yet....
$!
$	omi$cmdline_clear
$	omi$display_message "%OMI-?-BUG, you must provide some input... that's a bug..."
$	goto main$input_required_bug
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>
$!
$ main$_taglist:
$!
$	_tagblock = f$extract(0, f$locate("}", _variable) + 1, _variable)
$	_taglist = _variable - _tagblock
$	_tagblock = f$edit(_tagblock,"upcase") - "{TAG|" - "}"
$	if f$type ('_taglist') .eqs. "" then $ '_taglist' = ""
$	if f$type('_tagblock'$delimiter) .eqs. ""
$	   then
$		omi$display_message "%OMI-E-NODELIM, error in menu file - no delimiter for this taglist"
$		return
$	endif
$!
$	omi$screen taglist
$!
$ taglist$_prompt:
$!
$	if omi$_jumping
$	   then
$		_tag_sel = f$element(p3$_counter,",",p3)
$		if _tag_sel .eqs. "" .or. _tag_sel .eqs. ","
$		   then
$			read /end_of_file=main$end_taglist sys$command _tag_sel -
			   /prompt="[''screen$line_command';''screen$default_position'HSelect, <Ctrl/Z> when done: " 
$			if f$type(jump$_norefresh) .nes. "" then -
			   $ deletee /symbol /local jump$_norefresh
$			omi$_jumping = 0
$		   else $ p3$_counter = p3$_counter + 1
$		endif
$	   else $ read /end_of_file=main$end_taglist sys$command _tag_sel -
		   /prompt="[''screen$line_command';''screen$default_position'HSelect, <Ctrl/Z> when done: " 
$	endif
$	if f$edit(_tag_sel,"upcase") .eqs. "^Z" then $ goto main$end_taglist
$	omi$cmdline_clear
$	omi$msgline_clear
$!
$	if _tag_sel .eqs. "" then $ goto taglist$_prompt
$	if f$type(_tag_sel) .nes. "INTEGER"
$	   then
$		omi$display_message "%OMI-W-INTONLY, specify integers only"
$		if f$type(jump$_norefresh) .nes. "" then -
		   $ deletee /symbol /local jump$_norefresh
$		omi$_jumping = 0
$		goto taglist$_prompt
$	endif
$	_reverse = 0
$!
$	if f$type('_tagblock'$value'_tag_sel') .eqs. ""
$	   then
$		_sel_m_1 = _tag_sel - 1
$		if f$type('_tagblock'$value'_sel_m_1') .eqs. ""
$		   then
$			omi$display_message "%OMI-W-IVSEL, selection is out of range"
$			if f$type(jump$_norefresh) .nes. "" then -
			   $ deletee /symbol /local jump$_norefresh
$			omi$_jumping = 0
$			goto taglist$_prompt
$		   else $ _reverse = 1
$		endif
$	endif
$!
$	if _reverse then $ _tag_sel = 1
$!
$ tag$_reverse:
$!
$	_tag_value = '_tagblock'$value'_tag_sel' 
$	if f$locate(_tag_value,'_taglist') .lt. f$length('_taglist')
$	   then $ '_taglist' = '_taglist' - ('_tagblock'$value'_tag_sel' + '_tagblock'$delimiter)
$	   else $ '_taglist' = '_taglist' + '_tagblock'$value'_tag_sel'+ '_tagblock'$delimiter 
$	endif
$	omi$screen taglist '_tag_sel'
$!
$	if _reverse
$	   then
$		_tag_sel = _tag_sel + 1
$		if _tag_sel .le. _sel_m_1 then $ goto tag$_reverse
$	endif
$!
$	goto taglist$_prompt
$!
$ main$end_taglist:
$!
$	omi$refresh
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	The option was recognized as an internal OMI command, or an OMI
$!	command has been entered as input. Validate the command, and execute it.
$!
$ main$omi_command:
$!
$	omi$_p1 = f$edit(f$element(1, " ", omi$_command),"trim")
$	omi$_p2 = f$edit(f$element(2, " ", omi$_command),"trim")
$	omi$_p3 = f$edit(f$element(3, " ", omi$_command),"trim")
$	omi$_p4 = f$edit(f$element(4, " ", omi$_command),"trim")
$	omi$_p5 = f$edit(f$element(5, " ", omi$_command),"trim")
$	omi$_command = f$edit(f$element(0, " ", omi$_command),"trim")
$	_cmd_match = 0
$	_cmd_cnt = 0
$!
$ main$_validate_cmd:
$!
$	if f$element(_cmd_cnt, "#", omi$valid_commands) .eqs. "#" then -
	   $ goto main$end_validate_cmd
$	if omi$_command .eqs. f$extract(0, f$length(omi$_command), -
	   f$element(_cmd_cnt, "#", omi$valid_commands))
$	   then
$		omi$command = f$element(_cmd_cnt, "#", omi$valid_commands)
$		_cmd_match = _cmd_match + 1
$	endif
$!
$	_cmd_cnt = _cmd_cnt + 1
$	goto main$_validate_cmd
$!
$ main$end_validate_cmd:
$!
$	if _cmd_match .eq. 0
$	   then
$		omi$display_message "%OMI-W-IVCMD, invalid OMI command entered \''omi$_command'\"
$		return
$	endif
$!
$	if _cmd_match .ge. 2
$	   then
$		omi$display_message "%OMI-W-ABCMD, ambiguous OMI command entered \''omi$_command'\"
$		return
$	endif
$!
$	gosub main$execcmd_'omi$command'
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_back:
$!
$!==>	The OMI command BACK
$!
$	if 'omi$current_menu'$previous .eqs. "" then $ gosub main$_exit
$	omi$current_menu = 'omi$current_menu'$previous
$	omi$screen menu
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_exit:
$ main$execcmd_quit:
$!
$!==>	The OMI commands EXIT and QIUT
$!
$	gosub main$_exit
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_encrypt:
$!
$!==>	The OMI command ENCRYPT
$!
$	if 'omi$current_menu'$security_level .lt. 3
$	   then
$		omi$display_message "%OMI-W-NOPRIV, you are not authorized to encrypt a menu item"
$		return
$	endif
$!
$	if omi$_p1 .eqs. ""
$	   then
$		read /end_of_file=setcommand$_cancelled sys$command _encr_sect -
  		   /prompt="[''screen$line_command';''screen$default_position'H_From section: " 
$		omi$cmdline_clear
$	   else $ _encr_sect = omi$_p1
$	endif
$	_encr_sect = _encr_sect - "[" - "]"
$	if omi$_p2 .eqs. ""
$	   then
		$ read /end_of_file=setcommand$_cancelled sys$command _encr_item -
  		   /prompt="[''screen$line_command';''screen$default_position'H_Item: "
$		omi$cmdline_clear
$	   else $ _encr_item = omi$_p2
$	endif
$!
$	if f$type('_encr_sect'$'_encr_item') .eqs. ""
$	   then
$		omi$display_message "%OMI-W-NOSUCHITM, cannot find item \''_encr_item'\ or section \[''_encr_sect']\"
$		return
$	endif
$!
$	_encr = '_encr_sect'$'_encr_item'
$	omi$encrypt "''_encr'"
$	_encr = omi$encrypted
$	deletee /symbol /global omi$encrypted
$	omi$config 'omi$menu_file' update "''_encr_sect'" "''_encr_item'" "''_encr'"
$	if $status .eq. omi$_ok then -
	   $ omi$display_message "%OMI-S-ENCRYPT, successfully encrypted the menu item"
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_help:
$!
$!==>	The OMI command HELP
$!
$	assign /user /nolog TT: sys$input
$	help /library=Omi$:Omi$Menu
$	omi$refresh
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_increase:
$!
$!==>	The OMI command INCREASE
$!
$	if f$type('omi$current_menu'$counter) .eqs. ""
$	   then
$		omi$display_message "%OMI-W-NOCOUNT, this menu has no counter to increase"
$		return
$	endif
$!
$	_menu_counter = 'omi$current_menu'$counter
$	counter$'_menu_counter' == counter$'_menu_counter' + 1
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "REFRESH") -
	   then $ omi$refresh
$		
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_jump:
$!
$!==>	The OMI command JUMP
$!
$	if omi$_p1 .eqs. ""
$	   then
$		read /end_of_file=jump$_ignored sys$command omi$_p1 -
	 	   /prompt="[''screen$line_command';''screen$default_position'H_Submenu: "
$		omi$cmdline_clear
$		omi$msgline_clear
$		goto main$execcmd_jump
$	endif
$	assign sys$scratch:omi$jump_submenu._tmp$ sys$output
$	show symbol /global *$name
$	deassign sys$output
$	if .not. omi$_debug then -
	   $ set message /nofacility /noseverity /noidentification /notext
$	search sys$scratch:omi$jump_submenu._tmp$ "''omi$_p1'" -
	   /output=sys$scratch:omi$jump_submenu_found._tmp$
$	_status = $status
$	delete /nolog /noconfirm sys$scratch:omi$jump_submenu._tmp$;
$	if _status .eq. %X08D78053
$	   then
$		delete /nolog /noconfirm sys$scratch:omi$jump_submenu_found._tmp$;
$		omi$display_message "%OMI-W-NOSUCHNAME, no submenu found with name ''f$edit(omi$_p1,"upcase")'"
$		return
$	endif
$	open /read jump sys$scratch:omi$jump_submenu_found._tmp$
$	read jump _mnu_name
$	close jump
$	delete /nolog /noconfirm sys$scratch:omi$jump_submenu_found._tmp$;
$!
$	if f$type (omi$current_menu) .eqs. "" then $ omi$current_menu = ""
$	'f$edit(f$element(0, "$", _mnu_name), "collapse")'$previous = omi$current_menu
$	omi$current_menu = f$edit(f$element(0, "$", _mnu_name), "collapse")
$!
$	gosub main$check_security
$	if 'omi$current_menu'$security_level .eq. -1
$	   then
$		omi$msgline_clear
$		omi$display_message "%OMI-W-IVPWD, invalid password - access denied"
$		omi$current_menu = 'omi$current_menu'$previous
$		return
$	endif
$	if 'omi$current_menu'$security_level .eq. 0
$	   then
$		omi$msgline_clear
$		omi$display_message "%OMI-W-NOPRIV, you are not authorized for this menu"
$		omi$current_menu = 'omi$current_menu'$previous
$		return
$	endif
$	if f$type('omi$current_menu'$counter) .nes. ""
$	   then
$		_menu_counter = 'omi$current_menu'$counter
$		counter$'_menu_counter' == counter$'_menu_counter' + 1
$	endif
$!
$	if omi$_p2 .eqs. ""
$	   then $ omi$_jumping = 0
$	   else $ omi$_jumping = 1
$	endif
$	if f$type(jump$_norefresh) .eqs. "" then $ omi$refresh
$	return
$!
$ jump$_ignored:
$!
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_reset:
$!
$!==>	The OMI command RESET
$!
$	if omi$_p1 .eqs. ""
$	   then
$		read /end_of_file=resetcommand$_cancelled sys$command omi$_p1 -
		   /prompt="[''screen$line_command';''screen$default_position'H_What: " 
$		omi$_p1 = f$edit(omi$_p1,"uncomment,trim,compress,upcase")
$		omi$cmdline_clear
$		goto main$execcmd_reset
$	endif
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "COUNTER")
$	   then
$		if f$type('omi$current_menu'$counter) .eqs. ""
$		   then $ omi$display_message "%OMI-W-NOCOUNT, no counter available for this menu"
$		   else
$			_menu_counter = 'omi$current_menu'$counter
$			counter$'_menu_counter' == 0
$		endif
$		return
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "PASSWORD")
$	   then
$		if 'omi$current_menu'$security_level .lt. 3
$		   then
$			omi$display_message "%OMI-W-NOPRIV, you are not authorized to set a password"
$			return
$		endif
$		if f$type('omi$current_menu'$password) .eqs. ""
$		   then $ omi$display_message "%OMI-W-NOPWD, no password set on this menu"
$		   else
$			omi$config 'omi$menu_file' setcmd password reset
$			if $status .eq. omi$_ok then -
			   $ omi$display_message "%OMI-S-REMPWD, succesfully removed the password"
$		endif
$		return
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "NAME")
$	   then
$		if 'omi$current_menu'$security_level .lt. 3
$		   then
$			omi$display_message "%OMI-W-NOPRIV, you are not authorized to set the menu name"
$			return
$		endif
$		if f$type('omi$current_menu'$name) .eqs. ""
$		   then $ omi$display_message "%OMI-W-NONAME, no name set on this menu"
$		   else
$			omi$config 'omi$menu_file' setcmd name reset
$			if $status .eq. omi$_ok then -
			   $ omi$display_message "%OMI-S-REMNAME, succesfully removed the menu name"
$		endif
$		return
$	endif
$!
$	omi$display_message "%OMI-W-IVOPT, invalid RESET option"
$!
$ resetcommand$_cancelled:
$!
$	omi$cmdline_clear
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_set:
$!
$!==>	The OMI command SET
$!
$	if omi$_p1 .eqs. ""
$	   then
$		read /end_of_file=setcommand$_cancelled sys$command omi$_p1 -
		   /prompt="[''screen$line_command';''screen$default_position'H_What: " 
$		omi$_p1 = f$edit(omi$_p1,"uncomment,trim,compress,upcase")
$		omi$cmdline_clear
$		goto main$execcmd_set
$	endif
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "COUNTER")
$	   then
$		if omi$_p2 .eqs. ""
$		   then
$			read /end_of_file=setcommand$_cancelled sys$command omi$_p2 -
 			   /prompt="[''screen$line_command';''screen$default_position'H_Value: " 
$			omi$_p1 = f$edit(omi$_p1,"uncomment,trim,compress")
$			omi$cmdline_clear
$			goto main$execcmd_set
$		endif
$!
$		if f$type('omi$current_menu'$counter) .eqs. ""
$		   then $ omi$display_message "%OMI-W-IVCOUNT, no counter available for this menu"
$		   else
$			_menu_counter = 'omi$current_menu'$counter
$			counter$'_menu_counter' == omi$_p2
$		endif
$		return
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "PASSWORD")
$	   then
$		if 'omi$current_menu'$security_level .lt. 3
$		   then
$			omi$display_message "%OMI-W-NOPRIV, you are not authorized to set a password"
$			return
$		endif
$		set terminal /noecho
$!
$	  setpwd$_prompt:
$!
$		read /end_of_file=setpasswrd$_cancelled sys$command _pwd_1 -
 		   /prompt="[''screen$line_command';''screen$default_position'H_New password: " 
$		omi$cmdline_clear
$		if f$length(_pwd_1) .lt. 5
$		   then
$			omi$display_message "%OMI-W-SHORTPWD, password can not be shorter than 5 characters"
$			goto setpwd$_prompt
$		endif
$		read /end_of_file=setpasswrd$_cancelled sys$command _pwd_2 -
 		   /prompt="[''screen$line_command';''screen$default_position'H_Verification: " 
$		if _pwd_1 .nes. _pwd_2
$		   then
$			omi$display_message "%OMI-W-PWDVERFAIL, password verification failed"
$			goto setpasswrd$_cancelled
$		endif
$!
$		set terminal /echo
$		omi$encrypt "''_pwd_1'"
$		_new_password = omi$encrypted
$		deletee/symbol/global omi$encrypted
$		omi$config 'omi$menu_file' setcmd password "''_new_password'"
$		if $status .eq. omi$_ok then -
		   $ omi$display_message "%OMI-S-SETPWD, succesfully changed the password"
$		omi$cmdline_clear
$		return
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "NAME")
$	   then
$		if 'omi$current_menu'$security_level .lt. 3
$		   then
$			omi$display_message "%OMI-W-NOPRIV, you are not authorized to set the menu name"
$			return
$		endif
$		if omi$_p2 .eqs. ""
$		   then $ read /end_of_file=setcommand$_cancelled sys$command _new_name -
	 		   /prompt="[''screen$line_command';''screen$default_position'H_New name : " 
$		   else $ _new_name = omi$_p2
$		endif
$		omi$cmdline_clear
$!
$		omi$config 'omi$menu_file' setcmd name "''f$edit(_new_name,"collapse")'"
$		if $status .eq. omi$_ok then -
		   $ omi$display_message "%OMI-S-SETNAME, succesfully changed the menu name"
$		omi$cmdline_clear
$		return
$	endif
$!
$	omi$display_message "%OMI-W-IVOPT, invalid SET option"
$!
$ setcommand$_cancelled:
$!
$	omi$cmdline_clear
$	return
$!
$ setpasswrd$_cancelled:
$!
$	set terminal /echo
$	omi$cmdline_clear
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_show:
$!
$!==>	The OMI command SHOW
$!
$	if omi$_p1 .eqs. ""
$	   then
$		read /end_of_file=showcommand$_cancelled sys$command omi$_p1 -
		   /prompt="[''screen$line_command';''screen$default_position'H_What: " 
$		omi$_p1 = f$edit(omi$_p1,"uncomment,trim,compress,upcase")
$		omi$cmdline_clear
$		goto main$execcmd_show
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "COUNTER")
$	   then
$		if f$type('omi$current_menu'$counter) .nes. ""
$		   then
$			_menu_counter = 'omi$current_menu'$counter
$			omi$display_message "%OMI-I-COUNTER, counter for this menu is now ",counter$'_menu_counter'
$		   else $ omi$display_message "%OMI-I-NOCOUNTER, no counter available for this menu"
$		endif
$		return
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "VERSION")
$	   then
$		omi$display_message "%OMI-I-VERSION, current OMI version is ''omi$version'"
$		return
$	endif
$!
$	if f$length(omi$_p1) .ge. 3 .and. omi$_p1 .eqs. f$extract(0, f$length(omi$_p1), "NAME")
$	   then
$		if f$type('omi$current_menu'$name) .eqs. ""
$		   then $ omi$display_message "%OMI-I-NONAME, this menu has no name"
$		   else $ omi$display_message "%OMI-I-NAME, the name of this menu is \",f$edit('omi$current_menu'$name,"upcase"),"\"
$		endif
$		return
$	endif
$!
$	omi$display_message "%OMI-W-IVOPT, invalid SHOW option"
$!
$ showcommand$_cancelled:
$!
$	omi$cmdline_clear
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_main:
$!
$!==>	The OMI command MAIN
$!
$	omi$current_menu = "menu"
$	omi$screen menu
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_refresh:
$!
$!==>	The OMI command REFRESH
$!
$	omi$refresh
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_spawn:
$!
$!==>	The OMI command SPAWN
$!
$	cls
$	spawn 'omi$_p1' 'omi$_p2' 'omi$_p3' 'omi$_p4' 'omi$_p5'
$	ws ""
$  	read /end_of_file=spawn$_ignore sys$command dummy -
	   /prompt="[?25l''questions$wait_prompt' "
$!
$ spawn$_ignore:
$!
$	ws "[?25h"
$	omi$refresh
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_all:
$!
$!==>	The OMI command ALL
$!
$!	omi$cmdline_clear
$	if f$type('omi$current_menu'$input1) .eqs. ""
$	   then
$		omi$display_message "%OMI-W-NOINPUT, this menu contains no input fields"
$		return
$	endif
$!
$	_all_inputs = 1
$	_input = 1
$	_pointer = screen$default_position - 1
$!
$ main$_getall_inputs:
$!
$!	if _input .gt. 1 then $ ws "[''_line;''_pointer'H(0[1m~[0m(B"
$	_line = inputs$first_line - 1 + _input
$!	ws "[''_line;''_pointer'H(0[1m`[0m(B"
$	_variable = f$element(1,"#",'omi$current_menu'$input'_input')
$	if f$extract(0,5,f$edit(_variable,"upcase")) .eqs. "{TAG|"
$	   then
$		gosub main$_taglist
$		_input = _input + 1
$		if f$type('omi$current_menu'$input'_input') .nes. "" then -
		   $ goto main$_getall_inputs
$		return
$	endif
$	ws f$fao("[''_line';''inputs$value_location'H!''inputs$max_size'* ")
$	_sel_list = (f$extract(0,5,f$edit(_variable,"upcase")) .eqs. "{SEL|")
$	if _sel_list
$	   then
$		_select_list = f$extract(0, f$locate("}", _variable) + 1, _variable)
$		_variable = _variable - _select_list
$		_select_list = f$edit(_select_list,"upcase") - "{SEL|" - "}"
$		omi$screen select_list
$	endif
$!
$ allinput$_prompt:
$!
$	if _sel_list
$	   then
$		_prompt = f$element(0, "#",'omi$current_menu'$input'_input')
$		read /end_of_file=main$end_getall_inputs /prompt="[''screen$line_command';''screen$default_position'H''_prompt': " sys$command _value
$	   else $ read /end_of_file=main$end_getall_inputs /prompt="[''_line';''inputs$value_location'H" sys$command _value
$	endif
$!
$	if _value .eqs. ""
$	   then
$		if f$type('_variable') .eqs. ""
$		   then $ _value = main$empty_value
$		   else
$			_format = f$element(3, "#",'omi$current_menu'$input'_input') 
$			if _format .nes. "" .and. _format .nes. "#"
$			   then
$				gosub input$_format
$				omi$cmdline_clear
$				if .not. _frm_okay
$				   then 
$					ws f$fao("[''_line';''inputs$value_location'H!''inputs$max_size'* ")
$					goto allinput$_prompt
$				endif
$				_value = '_variable'
$			endif
$			_value = '_variable'
$		endif
$	   else
$		if _sel_list
$		   then
$			_selected = '_value'
$			if f$type(_selected) .nes. "INTEGER" .or. -
 			   f$type('_select_list'$value'_selected') .eqs. ""
$			   then
$				omi$display_message "%OMI-W-IVCHOICE, please specify a vaild choice"
$				omi$cmdline_clear
$				ws f$fao("[''_line';''inputs$value_location'H!''inputs$max_size'* ")
$				goto allinput$_prompt
$			endif
$			omi$refresh
$			'_variable' = '_select_list'$value'_selected'
$ 		   else
$			'_variable' = _value
$			_format = f$element(3, "#",'omi$current_menu'$input'_input') 
$			if _format .nes. "" .and. _format .nes. "#"
$			   then
$				gosub input$_format
$				omi$cmdline_clear
$				if .not. _frm_okay
$				   then 
$					ws f$fao("[''_line';''inputs$value_location'H!''inputs$max_size'* ")
$					goto allinput$_prompt
$				endif
$				_value = '_variable'
$			endif
$		endif
$	endif
$!
$	_blanks = inputs$max_size - f$length(_value) + 1
$	if f$length(_value) .le. inputs$max_size
$	   then $ ws f$fao("[''_line';''inputs$value_location'H''_value'!''_blanks'* ")
$	   else $ ws "[''_line';''inputs$value_location'H''f$extract(0,inputs$max_size,_value)'(0`(B"
$	endif
$	_input = _input + 1
$	if f$type('omi$current_menu'$input'_input') .nes. "" then -
	   $ goto main$_getall_inputs
$!
$ main$end_getall_inputs:
$!
$	if _sel_list then $ omi$refresh
$	return
$!
$!******************************************************************************
$!
$!==>	The OMI command (SILENT_)DCL
$!
$ main$execcmd_silent_dcl:
$!
$	_silent = 1
$	goto dclcommand$
$!
$ main$execcmd_dcl:
$!
$	_silent = 0
$	goto dclcommand$
$!
$ dclcommand$:
$!
$	if omi$_p1 .eqs. ""
$	   then
$		read /end_of_file=dclcommand$_cancelled sys$command omi$_p1 -
		   /prompt="[''screen$line_command';''screen$default_position'H''questions$dcl_command': " 
$		omi$_p1 = f$edit(omi$_p1,"uncomment,trim,compress")
$		if omi$_p1 .eqs. "" then $ goto  main$execcmd_dcl
$		omi$_p2 = f$edit(f$element(1," ",omi$_p1),"trim")
$		omi$_p3 = f$edit(f$element(2," ",omi$_p1),"trim")
$		omi$_p4 = f$edit(f$element(3," ",omi$_p1),"trim")
$		omi$_p5 = f$edit(f$element(4," ",omi$_p1),"trim")
$		omi$_p1 = f$edit(f$element(0," ",omi$_p1),"trim")
$	endif
$	if _silent
$	   then
$		assign /nolog "''main$silent_output'" sys$output
$		assign /nolog "''main$silent_output'" sys$error
$		goto dclcommand$do_it
$	endif
$	_window = ( f$extract(0,3,f$edit(screen$scroll_region,"upcase")) .eqs. "ENA")
$	if _window
$	   then $ omi$screen setup_scroll_region
$	   else $ cls
$	endif
$!
$ dclcommand$do_it:
$!
$	'omi$_p1' 'omi$_p2' 'omi$_p3' 'omi$_p4' 'omi$_p5'
$	if _silent
$	   then
$		_silent_status = $status
$		deassign sys$output
$		deassign sys$error
$		deletee /symbol /local _silent
$		omi$display_message f$message(_silent_status)
$		goto dclcommand$_cancelled
$	endif
$!
$	ws ""
$  	read /end_of_file=dclcommand$_ignore sys$command dummy -
	   /prompt="[?25l''questions$wait_prompt' "
$!
$ dclcommand$_ignore:
$!
$	ws "[?25h"
$	if _window
$	   then $ omi$screen erase_scroll_region
$	   else $ omi$refresh
$	endif
$!
$ dclcommand$_cancelled:
$!
$	omi$cmdline_clear
$	return
$!
$!******************************************************************************
$!
$ main$execcmd_manage:
$!
$!==>	The OMI command MANAGE
$!
$	if omi$_p1 .eqs. "BACK"
$	   then
$		omi$_p1 = omi$manage_started_from
$	   else
$		if f$edit(omi$menu_file,"upcase") .eqs. "OMI$MANAGE"
$		   then
$			omi$display_message "%OMI-I-MANAGE, you already are in the Management Menu"
$			return
$		endif
$		omi$manage_started_from = "''omi$menu_file'"
$		omi$_p1 = "OMI$MANAGE"
$	endif
$	goto main$execcmd_menu
$!
$!******************************************************************************
$!
$ main$execcmd_menu:
$!
$!==>	The OMI command MENU
$!
$	if omi$_p1 .nes. ""
$	   then
$		omi$new_menu_file = "''omi$_p1'"
$		omi$_p1 = ""
$	   else
$		read /end_of_file=newmenu$_ignored sys$command omi$new_menu_file -
	 	   /prompt="[''screen$line_command';''screen$default_position'HMenu file: "
$		omi$cmdline_clear
$		omi$msgline_clear
$	endif
$	omi$previous_menu_file = "''omi$menu_file'"
$	omi$menu_file = "''omi$new_menu_file'"
$	omi$config 'omi$menu_file
$	if $status .eq. omi$_warning
$	   then
$		omi$menu_file = "''omi$previous_menu_file'"
$		goto main$execcmd_menu 
$	endif
$	omi$config "''omi$previous_menu_file'" Cleanup
$	return
$!
$ newmenu$_ignored:
$!
$	omi$cmdline_clear
$	omi$msgline_clear
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	Find out is a securoty level is defined for the current menu. If so,
$!	act upon it. If there's no security, return the default value, which
$!	is '2', meaning read and exec access for all users.
$!	If the current user is the owner of this (sub) menu, the security
$!	level will allways be '3' (read, exec and write).
$!
$ main$check_security:
$!
$	if f$type('omi$current_menu'$owner) .nes. ""
$	   then $ _owner = 'omi$current_menu'$owner
$	   else
$		if f$type(menu$owner) .nes. ""
$		   then
$			_owner = menu$owner
$		endif
$	endif
$!
$	if f$type('omi$current_menu'$password) .nes. ""
$	   then
$		if 'omi$current_menu'$password .eqs. ""
$		   then
$			omi$encrypt "Omi$System"
$			'omi$current_menu'$password == omi$encrypted
$			deletee /symbol /global omi$encrypted
$		endif
$		on control_y then $ goto password$cancel_input
$		set terminal /noecho
$		read /end_of_file=password$cancel_input /prompt="[''screen$line_command';''screen$default_position'HPassword: " sys$command _password
$	        goto password$_encrypt
$!
$	   password$cancel_input:
$!
$		'omi$current_menu'$security_level = -1
$		on control_y then $ goto main$_interrupt
$		omi$cmdline_clear
$		set terminal /echo
$		return
$!
$	   password$_encrypt:
$!
$		on control_y then $ goto main$_interrupt
$		omi$cmdline_clear
$		set terminal /echo
$		omi$variable = "_password"
$		omi$input_validate
$		if $status .eq. omi$_error
$		   then
$			'omi$current_menu'$security_level = -1
$			return
$		endif
$		if f$length(_password) .lt. 5
$		   then
$			'omi$current_menu'$security_level = -1
$			return
$		endif
$		omi$encrypt "''_password'"
$		_encryptd = omi$encrypted
$		deletee/symbol/global omi$encrypted
$		if _encryptd .nes. 'omi$current_menu'$password
$		   then
$			'omi$current_menu'$security_level = -1
$			return
$		endif
$		deletee/symbol/local _encryptd
$!
$		if f$type('omi$current_menu'$password_level) .nes. ""
$		   then
$			'omi$current_menu'$security_level = 'omi$current_menu'$password_level
$			return
$		endif
$	endif
$!
$	if f$type(_owner) .nes. ""
$	   then
$		if f$locate(omi$current_user, f$edit(_owner,"upcase")) .lt. -
		   f$length(f$edit(_owner,"upcase"))
$		   then
$			'omi$current_menu'$security_level = 3
$			deletee/symbol/local _owner
$			return
$		endif
$	endif
$!
$	if f$type('omi$current_menu'$security) .eqs. ""
$	   then
$		'omi$current_menu'$security_level = 2
$		return
$	endif
$!
$	_security = 'omi$current_menu'$security
$!
$	if f$type('_security'$'omi$current_user') .nes. ""
$	   then $ _authorisation = f$edit('_security'$'omi$current_user',"upcase")
$	   else $ if f$type('_security'$all_users) .nes. "" then -
		    $ _authorisation = f$edit('_security'$all_users,"upcase")
$	endif
$!
$	if f$type(_authorisation) .eqs. ""
$	   then
$		'omi$current_menu'$security_level = 0
$		return
$	endif
$!
$	if f$locate("WRITE",_authorisation) .lt. f$length(_authorisation)
$	   then
$		deletee/symbol/local _authorisation
$		'omi$current_menu'$security_level = 3
$		return
$	endif
$!
$	if f$locate("EXEC",_authorisation) .lt. f$length(_authorisation)
$	   then
$		deletee/symbol/local _authorisation
$		'omi$current_menu'$security_level = 2
$		return
$	endif
$!
$	if f$locate("READ",_authorisation) .lt. f$length(_authorisation)
$	   then
$		deletee/symbol/local _authorisation
$		'omi$current_menu'$security_level = 1
$		return
$	endif
$!
$	deletee/symbol/local _authorisation
$	'omi$current_menu'$security_level = 0
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	The main exit routines. The cleanup procedure is called with a parameter
$!	that indicates whether or not the screen should be erased. In case or
$!	errors, we don't want to, since that would also erase the messages.
$!
$ main$_interrupt:
$!
$	if f$type(_silent) .nes. "" 
$	   then
$		if _silent      ! In case the interrupt was during SILENT_DCL
$		   then
$			deassign sys$output
$			deassign sys$error
$		endif
$	endif
$	omi$_cls = "NOCLS"
$	gosub main$_cleanup
$	ws "%OMI-S-BYEBYE, done!"
$	set message 'omi$_message'	! Just in case...
$	if .not. omi$_debug
$	   then $ if omi$_verify then $ set verify
$	endif
$	exit %X28
$!
$ main$_fatal:
$!
$	if f$type(_silent) .nes. "" 
$	   then
$		if _silent      ! In case the error was during SILENT_DCL
$		   then
$			deassign sys$output
$			deassign sys$error
$		endif
$	endif
$	omi$_cls = "NOCLS"
$	gosub main$_cleanup
$	ws "%OMI-S-BYEBYE, done!"
$	set message 'omi$_message'	! Just in case...
$	if .not. omi$_debug
$	   then $ if omi$_verify then $ set verify
$	endif
$	exit %X2c
$!
$ main$_exit:
$!
$	if f$type(menu$on_exit) .nes. ""
$	   then
$		omi$exit_job = menu$on_exit
$		if f$locate(".",omi$exit_job) .eq. f$length(omi$exit_job) -
		   then $ omi$exit_job = omi$exit_job + ".OMI"
$		@'omi$exit_job
$		if $status .eq. omi$_warning then $ return
$	endif
$	omi$_cls = "CLS"
$	gosub main$_cleanup
$	ws "%OMI-S-BYEBYE, done!"
$	set message 'omi$_message'	! Workaround.... there's a bug somewhere
$	if .not. omi$_debug
$	   then $ if omi$_verify then $ set verify
$	endif
$	exit 1
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	When this procedure initializes, following options are checked to
$!	find the start menu. If it ain't given as a parameter, the procedure
$!	looks for the file Omi$Menu.Mnu in the current directory. Next, a
$!	logical is checked to find a user specified default. Finally, the
$!	procedure looks for the file Omi$Menu.Mnu in Sys$Login.
$!	The order in which this takes place can be modified using the 
$!	'search_path' variable in the 'main' section of the configuration
$!	file, except for the parameter; this will allways overrule everything
$!	else.
$!
$ main$_getstart:
$!
$	if p1 .nes. ""
$	   then
$		omi$startmenu = "''f$parse(p1,,,"name")'.MNU"
$		return
$	endif
$!
$	if f$trnlnm("Omi$StartMenu") .nes. ""
$	   then
$		omi$startmenu = "''f$parse(f$trnlnm("Omi$StartMenu"),,,"name")'.MNU"
$		return
$	endif
$!
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	Setup the environment. This is done by calling the subprocedures
$!	that will setupo all global symbols, define the menus and
$!	the screen layout, and define some internally used foreign
$!	commands. Symbols defined in this routine can be local, since
$!	this is the master procedure.
$!
$ main$_initialize:
$!
$	omi$nodename = f$edit(f$getsyi("scsnode"),"collapse")
$	omi$current_user = f$edit(f$getjpi(0,"username"),"collapse")
$	omi$_ok        =    1
$	omi$_cancelled = 9901
$	omi$_warning   = 9903
$	omi$_error     = 9905
$	if f$trnlnm("Omi$") .eqs. ""
$	   then
$		_thisfile_location = -
		   f$parse(f$environment("procedure"),,,"device","no_conceal") + -
		   f$parse(f$environment("procedure"),,,"directory","no_conceal") - "]["
$		define /nolog Omi$ "''_thisfile_location'"
$	endif
$!
$	@Omi$:Omi$Config Setup
$	if $status .eq. omi$_error then $ exit %X2c
$	if f$type(main$version_id) .eqs. ""
$	   then
$ 		write sys$error "%OMI-W-OUTDATED, your configuration file is of an older version"
$ 		write sys$error "-OMI-I-UPDATE, it is recommended to update from OMI$:OMI$MENU.CFG"
$		read /end_of_file=init$_ignore_ctrlz /prompt="Press <Return> to continue " sys$command _dummy
$	   else
$		if main$version_id .nes. omi$version_id
$		   then
$	 		write sys$error "%OMI-W-OUTDATED, your configuration file is of an older version"
$ 			write sys$error "-OMI-I-UPDATE, it is recommended to update from OMI$:OMI$MENU.CFG"
$			read /end_of_file=init$_ignore_ctrlz /prompt="Press <Return> to continue " sys$command _dummy
$!
$		 init$_ignore_ctrlz:
$!
$			@Omi$:Omi$Config Setup "" Cleanup
$			exit
$		endif
$	endif
$!
$	@Omi$:Omi$Screen Setup
$!
$	omi$config       := "@Omi$:Omi$Config Menu"
$	omi$screen       := "@Omi$:Omi$Screen"
$	omi$refresh      := "@Omi$:Omi$Screen refresh"
$	omi$display_info := "@Omi$:Omi$Screen display_info"
$!
$	omi$cmdline_clear   := "write sys$output f$fao(""[''screen$line_command';''screen$default_position'H!''screen$line_length'* "")"
$	omi$msgline_clear   := "write sys$error f$fao(""[''screen$line_message';''screen$default_position'H!''screen$line_length'* "")"
$	omi$display_message := "write sys$error ""[''screen$line_message';''screen$default_position'H""+
$!
$	open /read /share=read /error=main$notoolbox_ini -
	   tb$init Omi$:Omi$ToolBox.Ini
$!
$ main$_init_toolbox:
$!
$	read /end_of_file=main$end_init_toolbox tb$init _tool
$	_tool = f$edit(_tool,"uncomment,collapse,upcase")
$	if _tool .eqs. "" then $ goto main$_init_toolbox
$	if _tool .eqs. "<EOF>" then $ goto main$end_init_toolbox
$	if f$extract(0,1,_tool) .eqs. "["
$	   then
$		toolbox = _tool - "[" - "]"
$		toolbox = f$parse(toolbox,"OMI$:",".COM")
$		goto main$_init_toolbox
$	endif
$	if toolbox .eqs. ""
$	   then
$		omi$display_message "%OMI-E-TOOLERR, no toolbox defined in \OMI$:OMI$TOOLBOX.INI\"
$		goto main$end_init_toolbox
$	endif
$	if .not. omi$_debug then -
	   $ set message /nofacility /noseverity /noidentification /notext
$	search 'toolbox' "''_tool'$:" /output=nla0:
$	if $status .eq. %X08D78053
$	   then
$		set message 'omi$_message
$		omi$display_message "%OMI-W-NOROUTINE, no routine found for the command ''_tool'"
$		read /end_of_file=mit$_no_routine sys$command _dummy -
 		   /prompt="[''screen$line_command';''screen$default_position'H[?25l''questions$wait_prompt' "
$		omi$msgline_clear
$		omi$cmdline_clear
$		omi$display_message "-OMI-I-CHECKTOOLBOX, check the toolbox command file"
$		read /end_of_file=mit$_no_routine sys$command _dummy -
 		   /prompt="[''screen$line_command';''screen$default_position'H''questions$wait_prompt' "
$		goto mit$_no_routine
$	endif
$!
$	set message 'omi$_message
$	omi$'_tool' := "@''toolbox' ''_tool'"
$	goto main$_init_toolbox
$!
$ mit$_no_routine:
$!
$	ws "[?25h"
$	omi$msgline_clear
$	omi$cmdline_clear
$	goto main$_init_toolbox
$!
$ main$end_init_toolbox:
$!
$	close tb$init
$!
$! Define the valid internal commands
$!
$	omi$valid_commands = "#ALL"		+ -
			     "#BACK"		+ -
			     "#DCL"		+ -
 			     "#ENCRYPT"		+ -
			     "#EXIT"		+ -
			     "#HELP"		+ -
			     "#INCREASE"	+ -
			     "#JUMP"		+ -
			     "#MAIN"		+ -
			     "#MANAGE"		+ -
			     "#MENU"		+ -
			     "#QUIT"		+ -
			     "#REFRESH"		+ -
			     "#RESET"		+ -
			     "#SET"		+ -
			     "#SILENT_DCL"	+ -
			     "#SHOW"		+ -
			     "#SPAWN"		+ -
			     "#"
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	Find out if any default values where defined for input items. If so,
$!	set them.
$!	This has to be done in this procedure, since the symbols have to
$!	be local.
$!
$ main$default_values:
$!
$	assign sys$scratch:omi$setup_defaults._tmp$ sys$output
$	show symbol /global *$input*
$	deassign sys$output
$	open /read omi$setup_defaults sys$scratch:omi$setup_defaults._tmp$
$!
$ init$_defaults:
$!
$	read /end_of_file=init$end_defaults omi$setup_defaults omi$value
$	if f$element(2, "#", omi$value) .nes. "#" .and. -
	   f$element(2, "#", omi$value) .nes. """
$	   then
$		if f$extract(0, 1, f$element(1,"#",omi$value)) .eqs. "{"
$		   then
$			_value = f$extract(f$locate("}",f$element(1,"#",omi$value))+1, f$length(f$element(1,"#",omi$value)), f$element(1,"#",omi$value)) 
$			_block = f$extract(1, f$locate("}", f$element(1,"#",omi$value))-1, f$element(1,"#", omi$value))
$			_block = f$edit(_block,"upcase") - "SEL|"
$			if f$extract(0,4,_block) .nes. "TAG|" then 
			   $ '_value' = '_block'$'f$element(2,"#",omi$value)'
$		   else
$			_default_value = f$element(2, "#", omi$value) - """
$			_variable = "_default_value"
$			_format = f$element(3, "#",omi$value) - """
$			if _format .nes. "" .and. _format .nes. "#" -
			   .and. '_variable' .nes. ""
$			   then
$				gosub input$_format
$				if .not. _frm_okay
$			  	   then $ 'f$element(1, "#", omi$value) = "Invalid default value"
$			  	   else $ 'f$element(1, "#", omi$value) = '_variable'
$				endif
$			   else $ 'f$element(1, "#", omi$value) = '_variable'
$			endif
$		endif
$	endif
$	goto init$_defaults
$!
$ init$end_defaults:
$!
$	close omi$setup_defaults
$  	delete/noconfirm/nolog sys$scratch:omi$setup_defaults._tmp$;
$	return
$!
$!******************************************************************************

$!******************************************************************************
$!
$!==>	Delete all symbols and all logicals that were set by this procedure,
$!	erase the screen (if not called from an error routine), and restore
$!	all original settings.
$!
$ main$_cleanup:
$!
$	@Omi$:Omi$Screen Cleanup "''omi$_cls'"
$	ws "%OMI-I-CLEARMEM, clearing memory..."
$	@Omi$:Omi$ToolBox Cleanup
$	if f$type(omi$menu_file) .nes. "" then $ omi$config "''omi$menu_file'" Cleanup
$	@Omi$:Omi$Config Setup "" Cleanup
$	if f$type(omi$inputs) .nes. "" then $ gosub cleanup$missing_items
$	if f$type(_thisfile_location) .nes. "" then $ deassign Omi$
$!
$	return
$!
$ cleanup$missing_items:
$!
$	assign sys$scratch:omi$symbol_cleanup._tmp$ sys$output
$	show symbol /global *$item*
$	deassign sys$output
$	open /read omi$symbol_cleanup sys$scratch:omi$symbol_cleanup._tmp$
$!
$ cleanup$_symbols:
$!
$	read /end_of_file=cleanup$end_symbols omi$symbol_cleanup omi$symbol
$	if f$locate("command#all", f$edit(omi$symbol,"lowercase")) .lt. -
	   f$length(omi$symbol) then $ deletee/symbol/global -
	   'f$element(0, "=", f$edit(omi$symbol,"collapse"))
$	goto cleanup$_symbols
$!
$ cleanup$end_symbols:
$!
$	close omi$symbol_cleanup
$	delete/noconfirm/nolog sys$scratch:omi$symbol_cleanup._tmp$;
$	deletee/symbol/global omi$inputs
$	return
$!
$!******************************************************************************
